##
# File:  ChemRefDataCvsUtils.py
# Date:  4-Feb-2013  J. Westbrook
#
# Updated:
#
# 4-Feb-2013 jdw  Generalized from class ChemRefDataCvsUtils() for all chemical reference data repositories.
# 7-Feb-2013 jdw  Add remove method
##
"""
Wrapper for utilities for managing chemical reference data repositories and sandboxes.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import sys
import os
import os.path
import shutil
import time
import logging

from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppCc
from wwpdb.io.cvs.CvsAdmin import CvsAdmin, CvsSandBoxAdmin
from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil
from wwpdb.io.locator.ChemRefPathInfo import ChemRefPathInfo

logger = logging.getLogger(__name__)


class ChemRefDataCvsUtils(object):

    """Wrapper for utilities for managing chemical reference data repositories and sandboxes."""

    #

    def __init__(self, reqObj, verbose=False, log=sys.stderr):
        """ """
        self.__verbose = verbose
        self.__lfh = log
        self.__debug = False
        #
        # Information injected from the request object -
        #
        self.__reqObj = reqObj
        # self.__topPath             = self.__reqObj.getValue("TopPath")
        # self.__topSessioPath       = self.__reqObj.getValue("TopSessionPath")
        #
        self.__sObj = self.__reqObj.getSessionObj()
        self.__sessionPath = self.__sObj.getPath()
        # self.__sessionRelativePath = self.__sObj.getRelativePath()
        # self.__sessionId           = self.__sObj.getId()
        #
        self.__siteId = self.__reqObj.getValue("WWPDB_SITE_ID")
        self.__cI = ConfigInfo(self.__siteId)
        self.__cIAppCc = ConfigInfoAppCc(self.__siteId)
        self.__sbTopPath = self.__cIAppCc.get_site_refdata_top_cvs_sb_path()
        self.__pI = ChemRefPathInfo(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        #
        self.__vc, self.__vcAd = self.__setupCvs()
        #

    def __setupCvs(self):
        #  Assign site specific chemical reference data CVS repository details --
        #
        cvsRepositoryHost = self.__cI.get("SITE_REFDATA_CVS_HOST")
        cvsRepositoryPath = self.__cI.get("SITE_REFDATA_CVS_PATH")
        #
        # Assign authentication details from the request environment -
        #
        # cvsUser = self.__reqObj.getValue("SITE_REFDATA_CVS_USER")
        # cvsPassword = self.__reqObj.getValue("SITE_REFDATA_CVS_PASSWORD")
        #
        cvsUser = self.__cI.get("SITE_REFDATA_CVS_USER")
        cvsPassword = self.__cI.get("SITE_REFDATA_CVS_PASSWORD")
        #
        vc = CvsSandBoxAdmin(tmpPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)
        vc.setRepositoryPath(host=cvsRepositoryHost, path=cvsRepositoryPath)
        vc.setAuthInfo(user=cvsUser, password=cvsPassword)
        vc.setSandBoxTopPath(self.__sbTopPath)
        vcAd = CvsAdmin(tmpPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)
        vcAd.setRepositoryPath(host=cvsRepositoryHost, path=cvsRepositoryPath)
        vcAd.setAuthInfo(user=cvsUser, password=cvsPassword)

        return vc, vcAd

    def getSandBoxTopPath(self):
        return self.__vc.getSandBoxTopPath()

    def syncBird(self):
        """Update the CVS repositories related to BIRD PRD, family and chemical component definitions."""
        #
        textList = []
        #
        cvsProjectName = self.__pI.assignCvsProjectName(repType="PRD")
        ok1, text = self.__vc.update(projectDir=cvsProjectName, prune=True)
        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(sync) CVS %s update output is: %s", cvsProjectName, text)
        textList.append(text)
        #
        cvsProjectName = self.__pI.assignCvsProjectName(repType="PRD_FAMILY")
        ok2, text = self.__vc.update(projectDir=cvsProjectName)
        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(sync) CVS %s update output is: %s", cvsProjectName, text)
        textList.append(text)
        #
        cvsProjectName = self.__pI.assignCvsProjectName(repType="PRDCC")
        ok3, text = self.__vc.update(projectDir=cvsProjectName)
        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(sync) CVS %s update output is:%s", cvsProjectName, text)
        textList.append(text)
        self.__vc.cleanup()
        #
        ok = ok1 and ok2 and ok3
        return (ok, textList)

    def checkoutChemCompSerial(self):
        """
        checkout the CVS repository for the CCD
        """
        textList = []
        cvsProjectName = self.__pI.assignCvsProjectName(repType="CC")
        ok, text = self.__vc.checkOut(projectPath=cvsProjectName)
        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(checkout) CVS %s update status is: %r", cvsProjectName, ok)
            # logger.info("+ChemRefDataCvsUtils(sync) CVS %s update output is:\n%s\n" % (cvsProjectName,text[:100]))
        textList.append(text[:100])
        #
        # self.__vc.cleanup()
        #
        return (ok, textList)

    def checkoutPRDSerial(self):
        """
        checkout the CVS repository for the PRD
        """
        textList = []
        cvsProjectName = self.__pI.assignCvsProjectName(repType="PRD")
        ok, text = self.__vc.checkOut(projectPath=cvsProjectName)
        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(checkout) CVS %s update status is: %r", cvsProjectName, ok)
            # logger.info("+ChemRefDataCvsUtils(sync) CVS %s update output is:\n%s\n" % (cvsProjectName,text[:100]))
        textList.append(text[:100])
        cvsProjectName = self.__pI.assignCvsProjectName(repType="PRD_FAMILY")
        ok, text = self.__vc.checkOut(projectPath=cvsProjectName)
        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(checkout) CVS %s update status is: %r", cvsProjectName, ok)
            # logger.info("+ChemRefDataCvsUtils(sync) CVS %s update output is:\n%s\n" % (cvsProjectName,text[:100]))
        textList.append(text[:100])
        cvsProjectName = self.__pI.assignCvsProjectName(repType="PRDCC")
        ok, text = self.__vc.checkOut(projectPath=cvsProjectName)
        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(checkout) CVS %s update status is: %r", cvsProjectName, ok)
            # logger.info("+ChemRefDataCvsUtils(sync) CVS %s update output is:\n%s\n" % (cvsProjectName,text[:100]))
        textList.append(text[:100])
        #
        # self.__vc.cleanup()
        #
        return (ok, textList)

    def syncChemCompSerial(self):
        """Update the CVS repositories related to the chemical component dicitonary."""
        #
        textList = []
        #
        cvsProjectName = self.__pI.assignCvsProjectName(repType="CC")
        ok, text = self.__vc.update(projectDir=cvsProjectName, prune=True)
        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(sync) CVS %s update status is: %r", cvsProjectName, ok)
            # logger.info("+ChemRefDataCvsUtils(sync) CVS %s update output is:\n%s\n" % (cvsProjectName,text[:100]))
        textList.append(text[:100])
        #
        # self.__vc.cleanup()
        #
        return (ok, textList)

    def syncChemComp(self, numProc=8):
        """Update the CVS repositories related to the chemical component dicitonary."""
        #
        startTime = time.time()
        #
        cvsProjectName = self.__pI.assignCvsProjectName(repType="CC")
        dataS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
        dataList = [(cvsProjectName, a, True) for a in dataS]

        # Extended CCD support
        ext_ccd = self.__cIAppCc.get_extended_ccd_supp()
        if ext_ccd:
            dataList += [(cvsProjectName, a + b, True) for a in dataS for b in dataS]

        #
        mpu = MultiProcUtil(verbose=self.__debug)
        mpu.set(workerObj=self.__vc, workerMethod="updateList")
        ok, failList, _resultList, diagList = mpu.runMulti(dataList=dataList, numProc=numProc)
        endTime = time.time()
        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(syncChemComp) CVS %s update status is: %r in %.2f seconds", cvsProjectName, ok, endTime - startTime)

            if len(failList) > 0:
                logger.info("+ChemRefDataCvsUtils(syncChemComp) diagnostics %r", failList)
        return (ok, diagList)

    def updateFile(self, filePath):
        """Update the input file within the appropriate repository.  File names must obey repository
        naming conventions.

        """
        #
        textList = []
        ok = False
        #
        if not self.checkFileName(filePath):
            textList.append("Unrecognized file name %s" % filePath)
            return ok, textList

        if not self.exists(filePath):
            textList.append("Cannot update nonexistent repository file %s" % filePath)
            return ok, textList

        #
        projName, relPath = self.__pI.getCvsProjectInfo(self.__pI.assignIdCodeFromFileName(filePath))

        dstPath = os.path.join(self.__vc.getSandBoxTopPath(), projName, relPath)
        # (pth, fn) = os.path.split(dstPath)

        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(updateFile) CVS update %s %s working repository path is %s", projName, relPath, dstPath)

        #
        try:
            # make sure we have the latest version
            self.__vc.checkOut(os.path.join(projName, relPath))
            shutil.copy2(filePath, dstPath)
            ok, text = self.__vc.commit(projName, relPath)

        except:  # noqa: E722 pylint: disable=bare-except
            text = "+ChemRefDataCvsUtils(updateFile) CVS update exception"
            if self.__verbose:
                logger.info("+ChemRefDataCvsUtils(updateFile) CVS update %s %s status %r output is: %s", projName, relPath, ok, text)
                logger.exception("Unknown failued in CVS update")

        textList.append(text)
        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(updateFile) CVS update %s %s status %r output is: %s", projName, relPath, ok, text)
        self.__vc.cleanup()
        #
        return ok, textList

    def addFile(self, filePath):
        """Add the input file to the appropriate repository.  File names must obey repository
        naming conventions.

        """
        #
        textList = []
        ok = False
        #
        if not self.checkFileName(filePath):
            textList.append("Unrecognized file name %s" % filePath)
            return ok, textList

        if self.exists(filePath):
            textList.append("Cannot add existing repository file name %s" % filePath)
            return ok, textList

        #
        projName, relPath = self.__pI.getCvsProjectInfo(self.__pI.assignIdCodeFromFileName(filePath))

        dstPath = os.path.join(self.__vc.getSandBoxTopPath(), projName, relPath)
        (pth, _fn) = os.path.split(dstPath)

        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(addFile) CVS add %s %s working repository path is %s", projName, relPath, dstPath)

        # add any missing containing directories for the destination file.
        try:
            if not os.access(pth, os.F_OK):
                os.makedirs(pth)
                head, _tail = os.path.split(relPath)
                self.__vc.add(projName, head)
            shutil.copy2(filePath, dstPath)
            self.__vc.add(projName, relPath)
            ok, text = self.__vc.commit(projName, relPath)

        except:  # noqa: E722 pylint: disable=bare-except
            text = "+ChemRefDataCvsUtils(addFile) CVS add exception"
            if self.__verbose:
                logger.info("+ChemRefDataCvsUtils(addFile) CVS add %s %s status %r output is: %s", projName, relPath, ok, text)
                logger.exception("Failure in CVS add")

        textList.append(text)
        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(addFile) CVS add %s %s status %r output is: %s", projName, relPath, ok, text)
        self.__vc.cleanup()
        #
        return ok, textList

    def history(self, idCode):
        """Return the version history for the input id code from the appropriate repository.

        Id codes must obey repository naming conventions.

        """
        #
        textList = []
        ok = False
        #
        projName, relPath = self.__pI.getCvsProjectInfo(idCode)
        if projName is None or relPath is None:
            return ok, textList

        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(history) CVS history id %s project %s  path %s", idCode, projName, relPath)
        #
        try:
            cvsPath = os.path.join(projName, relPath)
            ok, text = self.__vcAd.getHistory(cvsPath)
            textList.append(text)
        except:  # noqa: E722 pylint: disable=bare-except
            textList.append("CVS history exception")
            if self.__verbose:
                logger.info("+ChemRefDataCvsUtils(history) CVS history exception %s %s status %r output is: %s", projName, relPath, ok, textList)
                logger.exception("Failure in CVS history")

        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(remove) CVS history %s %s status %r output is: %s", projName, relPath, ok, textList)
        self.__vcAd.cleanup()
        #
        return ok, textList

    def checkoutRevisions(self, idCode):
        """Return the pathlist of checked out revisions for the input id code from the appropriate repository.

        Id codes must obey repository naming conventions.

        """
        #
        pathList = []
        revList = []
        ok = False
        #
        projName, relPath = self.__pI.getCvsProjectInfo(idCode)
        if projName is None or relPath is None:
            return ok, pathList

        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(checkoutRevisions) id %s project %s  path %s", idCode, projName, relPath)
        #
        try:
            cvsPath = os.path.join(projName, relPath)
            ok, revList = self.__vcAd.getRevisionList(cvsPath)

            (_pth, fn) = os.path.split(cvsPath)
            (_base, ext) = os.path.splitext(fn)

            for revId in revList:
                rId = revId[0]
                outPath = os.path.join(self.__sessionPath, idCode + "-" + rId + ext)
                ok, text = self.__vcAd.checkOutFile(cvsPath=cvsPath, outPath=outPath, revId=rId)
                if ok:
                    pathList.append(outPath)
                    logger.info("CVS checkout status %r output %s is: %s", ok, outPath, text)
        except:  # noqa: E722 pylint: disable=bare-except
            pathList.append("Revision checkout exception")
            if self.__verbose:
                logger.info("+ChemRefDataCvsUtils(checkoutRevisions) exception %s %s status %r output is: %s", projName, relPath, ok, pathList)
                logger.exception("Failure in CVS revision checkout")

        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(checkoutRevisions) done %s %s status %r output is: %s", projName, relPath, ok, pathList)
        self.__vcAd.cleanup()
        #
        return ok, pathList

    def remove(self, idCode):
        """Remove the reference file associated with the input id code from the appropriate repository.
        Id codes must obey repository naming conventions.

        """
        #
        text = ""
        textList = []
        ok = False
        #
        projName, relPath = self.__pI.getCvsProjectInfo(idCode)
        if projName is None or relPath is None:
            return ok, textList

        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(remove) CVS remove id %s project %s  path %s", idCode, projName, relPath)
        #
        try:
            ok1, text = self.__vc.remove(projName, relPath, saveCopy=True)
            textList.append(text)

            # Remove the containing directory if required.
            repType = self.__pI.getIdType(idCode)
            if repType == "CC":
                (pth, _fn) = os.path.split(relPath)
                ok2, text = self.__vc.removeDir(projName, pth)
                textList.append(text)
                ok = ok1 and ok2
            else:
                ok = ok1
        except:  # noqa: E722 pylint: disable=bare-except
            textList.append("+ChemRefDataCvsUtils(remove) CVS remove exception")
            if self.__verbose:
                logger.info("+ChemRefDataCvsUtils(remove) CVS remove %s %s status %r output is: %s", projName, relPath, ok, text)
                logger.exception("Failure in CVS remove")

        if self.__verbose:
            logger.info("+ChemRefDataCvsUtils(remove) CVS remove %s %s status %r output is: %s", projName, relPath, ok, textList)
        self.__vc.cleanup()
        #
        return ok, textList

    def exists(self, filePath):
        """Test if the input 'filePath' corresponds to an existing entry in one of
        CVS repositories.

        Return True if an existing entry is found or False otherwise
        """
        if (filePath is None) or (len(filePath) < 7):
            return False

        projName, relPath = self.__pI.getCvsProjectInfo(self.__pI.assignIdCodeFromFileName(filePath))
        projPath = self.__pI.getProjectPath(self.__pI.assignIdCodeFromFileName(filePath))
        if projName is None:
            return False

        # check existing for file within the repository sandbox -
        if os.access(os.path.join(projPath, relPath), os.R_OK):
            return True

        return False

    def checkFileName(self, filePath):
        """Test if the input 'filePath' corresponds to the naming conventions of on of the
        CVS repositories.

        Return True for success or False otherwise
        """
        if (filePath is None) or (len(filePath) < 7):
            return False
        projName, _relPath = self.__pI.getCvsProjectInfo(self.__pI.assignIdCodeFromFileName(filePath))
        return projName is not None



##
# File:  ChemRefSearchDef.py
# Date:  22-Apr-2010 J. Westbrook
#
# Update:
#  13-Jun-2017 jdw add left join semantics to query definitions
##
"""
Chemical reference search definition data  -

"""


class ChemRefSearchDef(object):
    # Supported Query Types and Features
    _queryTypeDict = {
        "BIRD_ENTITY": {"autocomplete": True, "service": "rdbms", "class": "entity"},
        "BIRD_INSTANCE": {"autocomplete": True, "service": "rdbms", "class": "instance"},
        "CCD_ENTITY": {"autocomplete": True, "service": "rdbms", "class": "entity"},
        "CCD_INSTANCE": {"autocomplete": True, "service": "rdbms", "class": "instance"},
        "INDEX_MATCH_SUBSTRING": {"autocomplete": False, "service": "index", "class": "entity"},
        "INDEX_MATCH_EXACT": {"autocomplete": False, "service": "index", "class": "entity"},
        "INDEX_MATCH_RANGE_VALUE_PAIR": {"autocomplete": False, "service": "index", "class": "entity"},
        "INDEX_MATCH_SIMILAR": {"autocomplete": False, "service": "index", "class": "entity"},
        "INDEX_FORMULA_SUBSET": {"autocomplete": False, "service": "index", "class": "entity"},
        "INDEX_FORMULA_EXACT": {"autocomplete": False, "service": "index", "class": "entity"},
        "INDEX_FORMULA_EXACT_SKIPH": {"autocomplete": False, "service": "index", "class": "entity"},
        "INDEX_FORMULA_BOUNDED": {"autocomplete": False, "service": "index", "class": "entity"},
        "RANGE_VALUE_PAIR": {"autocomplete": False, "service": "rdbms", "class": "entity"},
    }
    _displayTypeDict = {
        "birdSummaryResults": [
            ("pdbx_reference_molecule_list", "family_prd_id", "Family ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "prd_id", "PRD ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "name", "Name", "%s", "str", "50px"),
            ("pdbx_reference_molecule", "class", "Class", "%s", "str", "20px"),
            ("pdbx_reference_molecule", "type", "Type", "%s", "str", "15px"),
            ("pdbx_reference_molecule", "formula", "Formula", "%s", "str", "25px"),
            ("pdbx_reference_molecule", "formula_weight", "Formula weight", "%s", "str", "10px"),
            ("pdbx_reference_molecule", "chem_comp_id", "CC ID", "%s", "str", "8px"),
        ],
        "birdSummaryResultsSequence": [
            ("pdbx_reference_molecule_list", "family_prd_id", "Family ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "prd_id", "PRD ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "name", "Name", "%s", "str", "50px"),
            ("pdbx_reference_molecule", "class", "Class", "%s", "str", "20px"),
            ("pdbx_reference_molecule", "type", "Type", "%s", "str", "15px"),
            ("pdbx_reference_molecule", "formula", "Formula", "%s", "str", "25px"),
            ("pdbx_reference_molecule", "chem_comp_id", "CC ID", "%s", "str", "8px"),
            ("pdbx_reference_entity_sequence", "one_letter_codes", "Sequence", "%s", "str", "50px"),
        ],
        "birdSummaryResultsSubcomponents": [
            ("pdbx_reference_molecule_list", "family_prd_id", "Family ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "prd_id", "PRD ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "name", "Name", "%s", "str", "50px"),
            ("pdbx_reference_molecule", "class", "Class", "%s", "str", "20px"),
            ("pdbx_reference_molecule", "type", "Type", "%s", "str", "15px"),
            ("pdbx_reference_molecule", "formula", "Formula", "%s", "str", "25px"),
            ("pdbx_reference_molecule", "chem_comp_id", "CC ID", "%s", "str", "8px"),
            ("pdbx_reference_entity_subcomponents", "seq", "Subcomponents", "%s", "str", "50px"),
        ],
        "birdSummaryResultsSequenceTlc": [
            ("pdbx_reference_molecule_list", "family_prd_id", "Family ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "prd_id", "PRD ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "name", "Name", "%s", "str", "50px"),
            ("pdbx_reference_molecule", "class", "Class", "%s", "str", "20px"),
            ("pdbx_reference_molecule", "type", "Type", "%s", "str", "15px"),
            ("pdbx_reference_molecule", "formula", "Formula", "%s", "str", "25px"),
            ("pdbx_reference_molecule", "chem_comp_id", "CC ID", "%s", "str", "8px"),
            ("pdbx_reference_entity_sequence_list", "three_letter_codes", "Sequence", "%s", "str", "50px"),
        ],
        "birdSummaryResultsSourceOrganism": [
            ("pdbx_reference_molecule_list", "family_prd_id", "Family ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "prd_id", "PRD ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "name", "Name", "%s", "str", "50px"),
            ("pdbx_reference_molecule", "class", "Class", "%s", "str", "20px"),
            ("pdbx_reference_molecule", "type", "Type", "%s", "str", "15px"),
            ("pdbx_reference_molecule", "formula", "Formula", "%s", "str", "25px"),
            ("pdbx_reference_molecule", "chem_comp_id", "CC ID", "%s", "str", "8px"),
            ("pdbx_reference_entity_src_nat", "organism_scientific", "Organism Name", "%s", "str", "50px"),
        ],
        "birdSummaryResultsSynonym": [
            ("pdbx_reference_molecule_list", "family_prd_id", "Family ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "prd_id", "PRD ID", "%s", "str", "18px"),
            ("pdbx_reference_molecule", "name", "Name", "%s", "str", "50px"),
            ("pdbx_reference_molecule_synonyms", "ordinal", "Synonym Number", "%s", "str", "5px"),
            ("pdbx_reference_molecule_synonyms", "name", "Synonym", "%s", "str", "40px"),
            ("pdbx_reference_molecule", "class", "Class", "%s", "str", "20px"),
            ("pdbx_reference_molecule", "type", "Type", "%s", "str", "15px"),
            ("pdbx_reference_molecule", "formula", "Formula", "%s", "str", "25px"),
            ("pdbx_reference_molecule", "chem_comp_id", "CC ID", "%s", "str", "8px"),
        ],
        "OldbirdSummaryResultsInstancePublic": [
            ("cleanv1.pdbx_molecule_features", "prd_id", "PRD ID", "%s", "str", "18px"),
            ("cleanv1.pdbx_molecule_features", "Structure_ID", "PDB_ID", "%s", "str", "10px"),
            ("cleanv1.pdbx_molecule_features", "name", "Name", "%s", "str", "50px"),
            ("cleanv1.pdbx_molecule_features", "class", "Class", "%s", "str", "20px"),
            ("cleanv1.pdbx_molecule_features", "type", "Type", "%s", "str", "20px"),
            ("cleanv1.pdbx_molecule_features", "details", "Details", "%s", "str", "50px"),
        ],
        "birdSummaryResultsInstancePublic": [
            ("da_internal_combine.pdbx_molecule_features", "prd_id", "PRD ID", "%s", "str", "18px"),
            ("da_internal_combine.pdbx_molecule_features", "Structure_ID", "RCSB_ID", "%s", "str", "18px"),
            ("da_internal_combine.pdbx_molecule_features", "name", "Name", "%s", "str", "50px"),
            ("da_internal_combine.pdbx_molecule_features", "class", "Class", "%s", "str", "20px"),
            ("da_internal_combine.pdbx_molecule_features", "type", "Type", "%s", "str", "20px"),
            ("da_internal_combine.pdbx_molecule_features", "details", "Details", "%s", "str", "50px"),
            ("da_internal_combine.rcsb_status", "status_code", "Status", "%s", "str", "10px"),
        ],
        "birdSummaryResultsInstanceInternal": [
            ("da_internal_combine.pdbx_molecule_features", "prd_id", "PRD ID", "%s", "str", "18px"),
            ("da_internal_combine.pdbx_molecule_features", "Structure_ID", "RCSB_ID", "%s", "str", "18px"),
            ("da_internal_combine.pdbx_molecule_features", "name", "Name", "%s", "str", "50px"),
            ("da_internal_combine.pdbx_molecule_features", "class", "Class", "%s", "str", "20px"),
            ("da_internal_combine.pdbx_molecule_features", "type", "Type", "%s", "str", "20px"),
            ("da_internal_combine.pdbx_molecule_features", "details", "Details", "%s", "str", "50px"),
            ("da_internal_combine.rcsb_status", "status_code", "Status", "%s", "str", "10px"),
        ],
        "ccdSummaryResults": [
            ("chem_comp", "id", "ID", "%s", "str", "10px"),
            ("chem_comp", "name", "Name", "%s", "str", "50px"),
            ("chem_comp", "type", "Type", "%s", "str", "15px"),
            ("chem_comp", "formula", "Formula", "%s", "str", "25px"),
            ("chem_comp", "formula_weight", "Formula weight", "%s", "str", "10px"),
            ("chem_comp", "pdbx_synonyms", "Synonyms", "%s", "str", "20px"),
            ("chem_comp", "pdbx_subcomponent_list", "Subcomponents", "%s", "str", "40px"),
            ("chem_comp", "pdbx_release_status", "Status", "%s", "str", "10px"),
        ],
        "OldccdSummaryResultsInstanceRel": [
            ("cleanv1.chem_comp", "id", "CC ID", "%s", "str", "10px"),
            ("cleanv1.pdb_entry", "pdb_id", "PDB ID", "%s", "str", "10px"),
            ("cleanv1.pdb_entry", "Structure_ID", "DEP ID", "%s", "str", "10px"),
            ("cleanv1.pdb_entry", "method", "Method", "%s", "str", "50px"),
            ("cleanv1.pdb_entry", "title", "Title", "%s", "str", "50px"),
            ("cleanv1.pdb_entry", "date_of_RCSB_release", "Release", "%s", "str", "50px"),
            ("cleanv1.pdb_entry", "status_code", "Status", "%s", "str", "50px"),
        ],
        "ccdSummaryResultsInstanceRel": [
            ("da_internal_combine.chem_comp", "id", "CC ID", "%s", "str", "10px"),
            ("da_internal_combine.rcsb_status", "pdb_id", "PDB ID", "%s", "str", "10px"),
            ("da_internal_combine.rcsb_status", "Structure_ID", "Structure ID", "%s", "str", "10px"),
            ("da_internal_combine.rcsb_status", "exp_method", "Method", "%s", "str", "50px"),
            ("da_internal_combine.rcsb_status", "title", "Title", "%s", "str", "50px"),
            # ('da_internal_combine.rcsb_status', 'date_of_RCSB_release', 'Release', '%s', 'str', '50px'),
            ("da_internal_combine.rcsb_status", "status_code", "Status", "%s", "str", "50px"),
        ],
        "ccdSummaryResultsInstanceInternal": [
            ("da_internal_combine.chem_comp", "id", "CC ID", "%s", "str", "10px"),
            ("da_internal_combine.rcsb_status", "pdb_id", "PDB ID", "%s", "str", "10px"),
            ("da_internal_combine.rcsb_status", "Structure_ID", "Structure ID", "%s", "str", "10px"),
            ("da_internal_combine.rcsb_status", "exp_method", "Method", "%s", "str", "50px"),
            ("da_internal_combine.rcsb_status", "title", "Title", "%s", "str", "50px"),
            # ('da_internal_combine.rcsb_status', 'date_of_RCSB_release', 'Release', '%s', 'str', '50px'),
            ("da_internal_combine.rcsb_status", "status_code", "Status", "%s", "str", "50px"),
        ],
        "ccdIndexResults": [
            ("index", "ccId", "ID", "%s", "str", "10px"),
            ("index", "name", "Name", "%s", "str", "50px"),
            ("index", "type", "Type", "%s", "str", "15px"),
            ("index", "formula", "Formula", "%s", "str", "25px"),
            ("index", "formulaWeight", "Formula weight", "%s", "str", "10px"),
            ("index", "synonyms", "Synonyms", "%s", "str", "20px"),
            ("index", "subcomponentList", "Subcomponents", "%s", "str", "40px"),
            ("index", "releaseStatus", "Status", "%s", "str", "10px"),
        ],
    }

    _keyDict = {
        "pdbx_reference_molecule_family": ("family_prd_id"),
        "pdbx_reference_molecule_list": ("family_prd_id", "prd_id"),
        "pdbx_reference_molecule": ("prd_id"),
        "pdbx_reference_entity_list": ("prd_id"),
        "pdbx_reference_entity_nonpoly": ("prd_id"),
        "pdbx_reference_entity_link": ("prd_id"),
        "pdbx_reference_entity_poly_link": ("prd_id"),
        "pdbx_reference_entity_poly": ("prd_id"),
        "pdbx_reference_entity_poly_seq": ("prd_id"),
        "pdbx_reference_entity_sequence": ("prd_id"),
        "pdbx_reference_entity_sequence_list": ("prd_id"),
        "pdbx_reference_entity_src_nat": ("prd_id"),
        "pdbx_reference_entity_subcomponents": ("prd_id"),
        "pdbx_reference_molecule_details": ("family_prd_id", "prd_id"),
        "pdbx_reference_molecule_synonyms": ("family_prd_id", "prd_id"),
        "pdbx_reference_molecule_annotation": ("family_prd_id", "prd_id"),
        "pdbx_reference_molecule_features": ("family_prd_id", "prd_id"),
        "pdbx_reference_molecule_related_structures": ("family_prd_id"),
        "pdbx_prd_audit": ("prd_id"),
        "pdbx_family_prd_audit": ("family_prd_id"),
        # Bird inst
        "cleanv1.pdbx_molecule_features": ("Structure_ID"),
        "da_internal_combine.pdbx_molecule_features": ("Structure_ID"),
        "da_internal_combine.rcsb_status": ("Structure_ID"),
        # Chemical component
        "chem_comp": ("Component_ID"),
        "chem_comp_atom": ("Component_ID"),
        "chem_comp_bond": ("Component_ID"),
        "pdbx_chem_comp_audit": ("Component_ID"),
        "pdbx_chem_comp_descriptor": ("Component_ID"),
        "pdbx_chem_comp_identifier": ("Component_ID"),
        # Chemical component instances
        "cleanv1.pdb_entry": ("pdb_id"),
        "cleanv1.chem_comp": ("Structure_ID"),
        # 'da_internal_combine.rcsb_status': ('Structure_ID'),
        "da_internal_combine.chem_comp": ("Structure_ID"),
    }

    _searchTypeDict = {
        "BIRD_FAMILY_ID": {
            "queryColList": ["pdbx_reference_molecule_list.family_prd_id"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResults",
            "displayTitle": "",
        },
        "BIRD_PRD_ID": {
            "queryColList": ["pdbx_reference_molecule.prd_id"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResults",
            "displayTitle": "",
            "leftJoinTableList": ["pdbx_reference_molecule_list"],
        },
        "BIRD_NAME": {
            "queryColList": ["pdbx_reference_molecule.name"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResults",
            "displayTitle": "",
            "leftJoinTableList": ["pdbx_reference_molecule_list"],
        },
        "BIRD_SYNONYM": {
            "queryColList": ["pdbx_reference_molecule_synonyms.name"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResultsSynonym",
            "displayTitle": "",
            "leftJoinTableList": ["pdbx_reference_molecule_list"],
        },
        "BIRD_NAME_OR_SYNONYM": {
            "queryColList": ["pdbx_reference_molecule.name", "pdbx_reference_molecule_synonyms.name"],
            "logicalOp": "OR",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResultsSynonym",
            "displayTitle": "",
            "leftJoinTableList": ["pdbx_reference_molecule_list"],
        },
        "BIRD_CC_ID": {
            "queryColList": ["pdbx_reference_molecule.chem_comp_id"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResults",
            "displayTitle": "",
            "leftJoinTableList": ["pdbx_reference_molecule_list"],
        },
        "BIRD_TYPE": {
            "queryColList": ["pdbx_reference_molecule.type"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResults",
            "displayTitle": "",
            "leftJoinTableList": ["pdbx_reference_molecule_list"],
        },
        "BIRD_CLASS": {
            "queryColList": ["pdbx_reference_molecule.class"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResults",
            "displayTitle": "",
            "leftJoinTableList": ["pdbx_reference_molecule_list"],
        },
        "BIRD_FORMULA": {
            "queryColList": ["pdbx_reference_molecule.formula"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResults",
            "displayTitle": "",
            "leftJoinTableList": ["pdbx_reference_molecule_list"],
        },
        "BIRD_SEQ_ONE_LETTER_CODE": {
            "queryColList": ["pdbx_reference_entity_sequence.one_letter_codes"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResultsSequence",
            "displayTitle": "",
            "leftJoinTableList": ["pdbx_reference_molecule_list"],
        },
        "BIRD_SEQ_THREE_LETTER_CODE": {
            "queryColList": ["pdbx_reference_entity_sequence_list.three_letter_codes"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResultsSequenceTlc",
            "displayTitle": "",
            "leftJoinTableList": ["pdbx_reference_molecule_list"],
        },
        "BIRD_SOURCE_ORGANISN_NAME": {
            "queryColList": ["pdbx_reference_entity_src_nat.organism_scientific"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResultsSourceOrganism",
            "displayTitle": "",
            "leftJoinTableList": ["pdbx_reference_molecule_list"],
        },
        "BIRD_SUBCOMPONENTS": {
            "queryColList": ["pdbx_reference_entity_subcomponents.seq"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "PRD",
            "displayType": "birdSummaryResultsSubcomponents",
            "displayTitle": "",
        },
        "BIRDINST_PRD_ID_PUBLIC": {
            "queryColList": ["da_internal_combine.pdbx_molecule_features.prd_id"],
            "logicalOp": "AND",
            "extraConditions": ' (da_internal_combine.rcsb_status.status_code = "REL") ',
            "orderByList": [
                "da_internal_combine.pdbx_molecule_features.prd_id",
                "da_internal_combine.rcsb_status.status_code",
                "da_internal_combine.pdbx_molecule_features.Structure_ID",
            ],
            "resourceId": "DA_INTERNAL_COMBINE",
            "displayType": "birdSummaryResultsInstancePublic",
            "displayTitle": "BIRD instances in public entries",
        },
        "BIRDINST_PRD_ID_INTERNAL": {
            "queryColList": ["da_internal_combine.pdbx_molecule_features.prd_id"],
            "logicalOp": "AND",
            "extraConditions": ' (da_internal_combine.rcsb_status.status_code != "REL") ',
            "orderByList": [
                "da_internal_combine.pdbx_molecule_features.prd_id",
                "da_internal_combine.rcsb_status.status_code",
                "da_internal_combine.pdbx_molecule_features.Structure_ID",
            ],
            "resourceId": "DA_INTERNAL_COMBINE",
            "displayType": "birdSummaryResultsInstanceInternal",
            "displayTitle": "BIRD instances in internal entries",
        },
        "CCD_CC_ID": {
            "queryColList": ["chem_comp.Component_id"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "CC",
            "displayType": "ccdSummaryResults",
            "displayTitle": "",
        },
        "CCD_NAME": {
            "queryColList": ["chem_comp.name"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "CC",
            "displayType": "ccdSummaryResults",
            "displayTitle": "",
        },
        "CCD_SYNONYM": {
            "queryColList": ["chem_comp.pdbx_synonyms"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "CC",
            "displayType": "ccdSummaryResults",
            "displayTitle": "",
        },
        "CCD_NAME_OR_SYNONYM": {
            "queryColList": ["chem_comp.name", "chem_comp.pdbx_synonyms"],
            "logicalOp": "OR",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "CC",
            "displayType": "ccdSummaryResults",
            "displayTitle": "",
        },
        "CCD_FORMULA": {
            "queryColList": ["chem_comp.formula"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "CC",
            "displayType": "ccdSummaryResults",
            "displayTitle": "",
        },
        "CCD_FORMULA_WEIGHT": {
            "queryColList": ["chem_comp.formula_weight"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "CC",
            "displayType": "ccdSummaryResults",
            "displayTitle": "",
        },
        "CCD_SUBCOMPONENTS": {
            "queryColList": ["chem_comp.pdbx_subcomponent_list"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": "CC",
            "displayType": "ccdSummaryResults",
            "displayTitle": "",
        },
        "CCDINST_CC_ID_PUBLIC": {
            "queryColList": ["da_internal_combine.chem_comp.id"],
            "logicalOp": "AND",
            "extraConditions": ' (da_internal_combine.rcsb_status.status_code = "REL") ',
            "orderByList": ["da_internal_combine.chem_comp.id", "da_internal_combine.rcsb_status.status_code", "da_internal_combine.rcsb_status.Structure_ID"],
            "resourceId": "DA_INTERNAL_COMBINE",
            "displayType": "ccdSummaryResultsInstanceRel",
            "displayTitle": "Chemical component instances in public entries",
        },
        "CCDINST_CC_ID_INTERNAL": {
            "queryColList": ["da_internal_combine.chem_comp.id"],
            "logicalOp": "AND",
            "extraConditions": ' (da_internal_combine.rcsb_status.status_code != "REL") ',
            "orderByList": ["da_internal_combine.chem_comp.id", "da_internal_combine.rcsb_status.status_code", "da_internal_combine.rcsb_status.Structure_ID"],
            "resourceId": "DA_INTERNAL_COMBINE",
            "displayType": "ccdSummaryResultsInstanceInternal",
            "displayTitle": "Chemical component instances in internal entries",
        },
        "CCDIDX_FORMULA": {
            "queryColList": ["formula"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_FORMULA_BOUNDED": {
            "queryColList": ["formula"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_FORMULA_SUBSET": {
            "queryColList": ["formula"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_FORMULA_HEAVY_ATOM_EXACT": {
            "queryColList": ["formula"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_FORMULA_EXACT": {
            "queryColList": ["formula"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_FORMULA_WEIGHT_RANGE": {
            "queryColList": ["formulaWeight"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_INCHI": {
            "queryColList": ["InChI"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_INCHIKEY": {
            "queryColList": ["InChIKey"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_INCHIKEY14": {
            "queryColList": ["InChIKey14"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_SUBCOMPONENTS": {
            "queryColList": ["subcomponentList"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_SUBCOMPONENT_SUBSET": {
            "queryColList": ["subcomponentList"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_SMILES": {
            "queryColList": ["smilesList"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_SMILES_STEREO": {
            "queryColList": ["smilesList"],
            # 'queryColList': ['smilesStereo'],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
        "CCDIDX_SIMILAR_NAME": {
            "queryColList": ["nameList"],
            "logicalOp": "AND",
            "extraConditions": "",
            "orderByList": [],
            "resourceId": None,
            "displayType": "ccdIndexResults",
            "displayTitle": "",
        },
    }



##
# File:  ChemRefDataMiscUtils.py
# Date:  16-Mar-2016  Migrated from cc_dict_utils  - J. Westbrook
#
# Updated:
#  18-Mar-2016  jdw adjust mode on data products to g+rw
#  18-Mar-2016  jdw add extra newline between concatenated components
#
"""
Wrapper for utilities for creating and maintaining various resource files containing
chemical reference data content stored in repositories and sandboxes.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import sys
import os
import os.path
import shutil
import time
import scandir
import filecmp
import fnmatch
import logging

from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppCc

from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil

from mmcif_utils.bird.PdbxPrdIo import PdbxPrdIo
from mmcif_utils.bird.PdbxPrdCcIo import PdbxPrdCcIo
from mmcif_utils.bird.PdbxFamilyIo import PdbxFamilyIo

from mmcif_utils.chemcomp.PdbxChemCompIo import PdbxChemCompIo

from wwpdb.utils.dp.RcsbDpUtility import RcsbDpUtility
from wwpdb.io.file.DataFile import DataFile

from wwpdb.utils.cc_dict_util.persist.PdbxChemCompDictUtil import PdbxChemCompDictUtil
from wwpdb.utils.cc_dict_util.persist.PdbxChemCompDictIndex import PdbxChemCompDictIndex

logger = logging.getLogger(__name__)


class ChemRefDataMiscUtils(object):
    """Wrapper for utilities for creating and maintaining various resource files containing
    chemical reference data content stored in repositories and sandboxes.
    """

    def __init__(self, reqObj, verbose=False, log=sys.stderr):
        """ """
        self.__verbose = verbose
        self.__lfh = log
        self.__debug = False
        #
        # Information injected from the request object -
        #
        self.__reqObj = reqObj
        #
        self.__sObj = self.__reqObj.getSessionObj()
        self.__sessionPath = self.__sObj.getPath()
        #

        self.__siteId = self.__reqObj.getValue("WWPDB_SITE_ID")
        self.__cI = ConfigInfo(self.__siteId)
        self.__cIConfigInfoCc = ConfigInfoAppCc(self.__siteId)
        self.__sbTopPath = self.__cIConfigInfoCc.get_site_refdata_top_cvs_sb_path()
        self.__projName = self.__cI.get("SITE_REFDATA_PROJ_NAME_CC")
        self.__ccDictPath = self.__cIConfigInfoCc.get_site_cc_dict_path()
        #
        self.__pathCCDict = self.__cIConfigInfoCc.get_cc_dict()
        self.__pathCCPathList = self.__cIConfigInfoCc.get_cc_path_list()
        self.__pathCCIdList = self.__cIConfigInfoCc.get_cc_id_list()
        self.__pathCCDictSerial = self.__cIConfigInfoCc.get_cc_dict_serial()
        self.__pathCCDictIdx = self.__cIConfigInfoCc.get_cc_dict_idx()
        #
        if sys.version_info[0] > 2:
            self.__pathCCDb = self.__cIConfigInfoCc.get_cc_db()
        else:
            self.__pathCCDb = os.path.join(self.__ccDictPath, "chemcomp.db")
        self.__pathCCIndex = self.__cIConfigInfoCc.get_cc_index()
        self.__pathCCParentIndex = self.__cIConfigInfoCc.get_cc_parent_index()

        self.__pathPrdChemCompCVS = self.__cIConfigInfoCc.get_site_prdcc_cvs_path()
        self.__pathPrdDictRef = self.__cIConfigInfoCc.get_site_prd_dict_path()
        self.__pathPrdDictFile = self.__cIConfigInfoCc.get_prd_dict_file()
        self.__pathPrdDictSerial = self.__cIConfigInfoCc.get_prd_dict_serial()
        self.__pathPrdCcFile = self.__cIConfigInfoCc.get_prd_cc_file()
        self.__pathPrdCcSerial = self.__cIConfigInfoCc.get_prd_cc_serial()
        self.__pathPrdSummary = self.__cIConfigInfoCc.get_prd_summary_cif()
        self.__pathPrdSummarySerial = self.__cIConfigInfoCc.get_prd_summary_sdb()
        self.__pathPrdFamilyMapping = self.__cIConfigInfoCc.get_prd_family_mapping()
        #
        self.__makeTopPaths()

    def __makeTopPaths(self):
        for top_path in (self.__ccDictPath, self.__pathPrdDictRef, self.__pathPrdChemCompCVS):
            if not os.path.exists(top_path):
                os.makedirs(top_path)

    def getBirdPathList(self):
        """Get pathlist for BIRD PRD and PRD Family data."""
        logger.info("+ChemRefDataLoad(getBirdPathList) Starting at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
        try:
            #
            birdCachePath = self.__cIConfigInfoCc.get_site_prd_cvs_path()
            birdFamilyCachePath = self.__cIConfigInfoCc.get_site_family_cvs_path()
            birdCcCachePath = self.__cIConfigInfoCc.get_site_prdcc_cvs_path()
            #
            #
            prd = PdbxPrdIo(verbose=self.__verbose, log=self.__lfh)
            prd.setCachePath(birdCachePath)
            pathList = prd.makeDefinitionPathList()
            #
            prdFam = PdbxFamilyIo(verbose=self.__verbose, log=self.__lfh)
            prdFam.setCachePath(birdFamilyCachePath)
            familyPathList = prdFam.makeDefinitionPathList()
            #
            prdCc = PdbxPrdCcIo(verbose=self.__verbose, log=self.__lfh)
            prdCc.setCachePath(birdCcCachePath)
            prdCcPathList = prdCc.makeDefinitionPathList()
            #
            return pathList, familyPathList, prdCcPathList
        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("In getBirdPathList")

        logger.info("+ChemRefDataLoad(getBirdPathList) Completed at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
        return [], [], []

    def __atomicRename(self, srcPath, dstPath, suffix=".old"):
        """Performs an atomic rename - while keeping the old file open to handle race conditions.
        On NFS, if you have a file open on server B, and server A moves (atomically) a new file onto another, the
        old file is unlinked by server.  Server B wlll then have a stale file handle for the already open file.

        The solution here is to hardlink the old file to a tempoerary, and then atomically move new to old -- the old
        inode is still valid until removed

        srcPath and dstPath must be on the same filesyetem.
        """

        tempPath = dstPath + suffix

        # Remove from previous round
        if os.path.exists(tempPath):
            os.remove(tempPath)

        if os.path.exists(dstPath):
            os.link(dstPath, tempPath)

        os.rename(srcPath, dstPath)

    def __makeTempPath(self, inpPath):
        try:
            pid = str(os.getpid())
            return inpPath + pid
        except:  # noqa: E722 pylint: disable=bare-except
            return inpPath

    def writeList(self, myList, outPath, mode=0o664):  # pylint: disable=unused-argument
        """ """
        try:
            tPath = self.__makeTempPath(outPath)
            ofh = open(tPath, "w")
            for el in myList:
                ofh.write("%s\n" % el)
            ofh.close()
            os.chmod(tPath, 0o664)
            shutil.move(tPath, outPath)
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            return False

    def concatPathList(self, pathList, outPath, mode=0o664):
        """ """
        (ok, _update) = self.concatPathListExt(pathList, outPath, mode)
        return ok

    def concatPathListExt(self, pathList, outPath, mode=0o664, avoidUpdate=False):  # pylint: disable=unused-argument
        """
        Concatenates files in pathList. If avoidUpdate is set, compare temporary file
        vs. destination.

        Returns (status, updated) - status if concatenation good. If True, updated will be
        set of file moved into place.
        """
        try:
            tPath = self.__makeTempPath(outPath)
            with open(tPath, "wb") as outfile:
                for f in pathList:
                    with open(f, "rb") as infile:
                        outfile.write(infile.read())
                        # to handle missing trailing newlines =
                        outfile.write(b"\n")
            os.chmod(tPath, 0o664)

            if avoidUpdate and os.path.exists(outPath):
                if filecmp.cmp(tPath, outPath, shallow=False):
                    os.unlink(tPath)
                    return (True, False)

            shutil.move(tPath, outPath)
            return (True, True)
        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("In concatPathListExt")
            return (False, False)

    def updateChemCompSupportFiles(self, idMaxLen=5, numProc=4, skipIndex=False):
        """Create full idlist, pathlist, concatenated chemical component dictionary file,
        serialized dictionary, and dictionary index.
        If skipIndex is set, will avoid serialization and indexing operations for testing
        """
        ok1 = ok2 = ok3 = False
        pathList = self.getChemCompPathListMulti(numProc=numProc)
        idList = []
        for pth in pathList:
            (_dn, fn) = os.path.split(pth)
            (ccId, ext) = os.path.splitext(fn)
            if (len(ccId) <= idMaxLen) and (ext == ".cif"):
                idList.append(ccId)
        self.writeList(idList, self.__pathCCIdList)
        self.writeList(pathList, self.__pathCCPathList)
        ok1 = self.concatPathList(pathList, self.__pathCCDict)
        #
        if skipIndex:
            ok2 = True
            ok3 = True
        else:
            ok2 = self.__serializeDictOp()
            ok3 = self.__indexDictOp()
        return ok1 and ok2 and ok3

    def updateChemCompPySupportFiles(self):
        """Create BSD DB of full chemical component dictionary, PRD CC files, index and parent index pickle files."""
        ok = False
        startTime = time.time()
        logger.info("Starting at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
        try:
            dU = PdbxChemCompDictUtil(verbose=self.__verbose, log=self.__lfh)
            logger.info("Starting full load at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

            ok = dU.makeStoreFromFile(dictPath=self.__pathCCDict, storePath=self.__pathCCDb)
            logger.info("Finished full load at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

            # We skip the next - as it incrementally loads the CC definitions into database which is good for low memory machines
            # but terribly slow
            # ok = self.updatePrdCCFiles()
            ok = True
            if ok:
                dIndx = PdbxChemCompDictIndex(verbose=self.__verbose, log=self.__lfh)
                dIndx.makeIndex(storePath=self.__pathCCDb, indexPath=self.__pathCCIndex)
                _pD, _cD = dIndx.makeParentComponentIndex(storePath=self.__pathCCDb, indexPath=self.__pathCCParentIndex)
                ok = True
            else:
                ok = False
        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("In updateChemCompPySupportFiles")
            ok = False

        endTime = time.time()
        logger.info("Completed at %s (%d seconds)", time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - startTime)
        return ok

    def __getPathList(self, topPath, pattern="*", excludeDirs=None, recurse=True):
        """Return a list of file paths in the input topPath which satisfy the input search criteria.

        This version does not follow symbolic links.
        """
        if excludeDirs is None:
            excludeDirs = []

        pathList = []
        #
        try:
            names = os.listdir(topPath)
        except os.error:
            return pathList

        # expand pattern
        pattern = pattern or "*"
        patternList = pattern.split(";")

        for name in names:
            fullname = os.path.normpath(os.path.join(topPath, name))
            # check for matching files
            for pat in patternList:
                if fnmatch.fnmatch(name, pat):
                    if os.path.isfile(fullname):
                        pathList.append(fullname)
                        continue
            if recurse:
                # recursively scan directories
                if os.path.isdir(fullname) and not os.path.islink(fullname) and (name not in excludeDirs):
                    pathList.extend(self.__getPathList(topPath=fullname, pattern=pattern, excludeDirs=excludeDirs, recurse=recurse))

        return pathList

    def updatePrdCCFiles(self):
        """Update persistent store from a path list of PRD chemical component defintions."""
        ok = False
        startTime = time.time()
        logger.info("Starting at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
        try:
            ccPathList = self.__getPathList(topPath=self.__pathPrdChemCompCVS, pattern="*.cif", excludeDirs=["CVS", "REMOVED", "FULL"])
            if self.__verbose:
                logger.info("PRD CC pathlist length is %d", len(ccPathList))
            #
            dUtil = PdbxChemCompDictUtil(verbose=self.__verbose, log=self.__lfh)
            logger.info("Starting incremental load at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
            dUtil.updateStoreByFile(pathList=ccPathList, storePath=self.__pathCCDb)
            logger.info("Finished incremental load at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
            ok = True
            #
        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("updatePrdCCFiles")

        endTime = time.time()
        logger.info("Completed at %s (%d seconds)", time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - startTime)
        return ok

    def __indexDictOp(self, minSize=10):
        """Make chemical component dictionary index from serialized dictionary."""
        logger.info("Starting")
        startTime = time.time()
        try:
            outPathTmp = self.__makeTempPath(self.__pathCCDictIdx)
            inpPath = self.__pathCCDictSerial
            logPath = os.path.join(self.__ccDictPath, "chem-comp-dict-makeindex.log")
            dp = RcsbDpUtility(tmpPath=self.__sessionPath, siteId=self.__siteId, verbose=self.__verbose)
            #
            dp.setDebugMode(flag=self.__debug)
            dp.imp(inpPath)
            dp.op("chem-comp-dict-makeindex")
            dp.expLog(logPath)
            dp.exp(outPathTmp)
            fSize = dp.expSize()
            if fSize > minSize:
                os.chmod(outPathTmp, 0o664)
                self.__atomicRename(outPathTmp, self.__pathCCDictIdx)
                os.chmod(self.__pathCCDictIdx, 0o664)
                ok = True
            else:
                ok = False
            if not self.__debug:
                dp.cleanup()
            endTime0 = time.time()
            if self.__verbose:
                logger.info("Dictionary index file size %d  in %.2f seconds", fSize, endTime0 - startTime)
            return ok
        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("Failure in __indexDictOp")
        return False

    def __serializeDictOp(self, minSize=10):
        """Serialize chemical component dictionary from concatenated dictionary text."""
        logger.info("Starting")
        startTime = time.time()
        try:
            outPathTmp = self.__makeTempPath(self.__pathCCDictSerial)
            inpPath = self.__pathCCDict
            logPath = os.path.join(self.__ccDictPath, "chem-comp-dict-serialize.log")
            dp = RcsbDpUtility(tmpPath=self.__sessionPath, siteId=self.__siteId, verbose=self.__verbose)
            #
            dp.setDebugMode(flag=self.__debug)
            dp.imp(inpPath)
            dp.op("chem-comp-dict-serialize")
            dp.expLog(logPath)
            dp.exp(outPathTmp)
            fSize = dp.expSize()
            if fSize > minSize:
                os.chmod(outPathTmp, 0o664)
                self.__atomicRename(outPathTmp, self.__pathCCDictSerial)
                os.chmod(self.__pathCCDictSerial, 0o664)
                ok = True
            else:
                ok = False
            if not self.__debug:
                dp.cleanup()
            endTime0 = time.time()
            if self.__verbose:
                logger.info("Serialized dictionary file size %d  in %.2f seconds", fSize, endTime0 - startTime)
            return ok
        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("Failure in __serializeDictOp")
        return False

    def getChemCompPathList(self):
        startTime = time.time()
        logger.info("+ChemRefDataDbUtils(getChemCompPathList) Starting at %s", time.strftime("%Y %m %d %H:%M:%S", time.localtime()))
        try:
            chemCompCachePath = self.__cIConfigInfoCc.get_site_cc_cvs_path()
            #
            #
            cc = PdbxChemCompIo(verbose=self.__verbose, log=self.__lfh)
            cc.setCachePath(chemCompCachePath)
            pathList = cc.makeComponentPathList()
            endTime0 = time.time()
            if self.__verbose:
                logger.info("Path list length %d  in %.2f seconds", len(pathList), endTime0 - startTime)
            return pathList
        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("In getChemCompPathLisr")

        endTime = time.time()
        logger.info("+ChemRefDataDbUtils(getChemCompPathList) Completed at %s (%.2f seconds)", time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - startTime)
        return []

    def _makeComponentPathListMulti(self, dataList, procName, optionsD, workingDir):  # pylint: disable=unused-argument
        """Return the list of chemical component definition file paths in the current repository."""
        pathList = []
        for subdir in dataList:
            dd = os.path.join(self.__sbTopPath, self.__projName, subdir)
            for root, _dirs, files in scandir.walk(dd, topdown=False):
                if "REMOVE" in root:
                    continue
                for name in files:
                    if name.endswith(".cif") and len(name) <= 9:  # Extends to support 5 character CCD
                        pathList.append(os.path.join(root, name))

        return dataList, pathList, []

    def getChemCompPathListMulti(self, numProc=8):
        """Return path list for all chemical component definition data files - (multiprocessing version)"""
        if self.__verbose:
            logger.info("Starting")
        startTime = time.time()
        try:
            dataS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
            dataList = [a for a in dataS]

            # Extended CCD support
            ext_ccd = self.__cIConfigInfoCc.get_extended_ccd_supp()
            if ext_ccd:
                dataList += [(a + b) for a in dataS for b in dataS]

            mpu = MultiProcUtil(verbose=True)
            mpu.set(workerObj=self, workerMethod="_makeComponentPathListMulti")
            _ok, _failList, retLists, _diagList = mpu.runMulti(dataList=dataList, numProc=numProc, numResults=1)
            pathList = retLists[0]
            endTime0 = time.time()
            if self.__verbose:
                logger.info("Path list length %d  in %.2f seconds", len(pathList), endTime0 - startTime)

            return pathList
        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("In getChemCompPathListMulti")

        endTime = time.time()
        logger.info("Completed at %s (%.2f seconds)", time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - startTime)
        return []

    def updatePrdSupportFiles(self, idMaxLen=14, numProc=4):  # pylint: disable=unused-argument
        """Create full idlist, pathlist, concatenated PRD dictionary file,
        serialized dictionary, and dictionary index.
        """
        startTime = time.time()
        ok1 = ok2 = ok3 = ok4 = ok5 = ok6 = False
        pathList, familyPathList, ccPathList = self.getBirdPathList()

        #        self.writeList(idList, self.__pathCCIdList)
        #        self.writeList(pathList, self.__pathCCPathList)
        (ok1, updated) = self.concatPathListExt(pathList, self.__pathPrdDictFile, avoidUpdate=True)
        # If prd file updated, then need serial
        if ok1 and updated:
            ok2 = self.__serializePrdDictOp("prd")
        else:
            ok2 = True

        (ok3, updated) = self.concatPathListExt(ccPathList, self.__pathPrdCcFile, avoidUpdate=True)
        if ok3 and updated:
            ok4 = self.__serializePrdDictOp("prdcc")
        else:
            ok4 = True

        # PRD summary files
        ok5 = self.__generatePrdSummaryOp()

        # Get family mappings
        ok6 = self.__generatePrdFamilyMapOp(familyPathList)

        endTime = time.time()
        logger.info("Completed  at %s (%.2f seconds)", time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - startTime)
        #        #
        #        ok2 =
        #        ok3 = self.__indexDictOp()
        #        return ok1 and ok2 and ok3
        return ok1 and ok2 and ok3 and ok4 and ok5 and ok6

    def __serializePrdDictOp(self, which, minSize=10):
        """Serialize chemical component dictionary from concatenated dictionary text."""
        logger.info("Starting %s", which)
        startTime = time.time()

        fList = {
            "prd": [self.__pathPrdDictFile, self.__pathPrdDictSerial, "prd-dict-serialize.log"],
            "prdcc": [self.__pathPrdCcFile, self.__pathPrdCcSerial, "prdcc-dict-serialize.log"],
        }
        try:
            ref = fList[which]
            outPathTmp = self.__makeTempPath(ref[1])
            inpPath = ref[0]
            logPath = os.path.join(self.__pathPrdDictRef, ref[2])
            dp = RcsbDpUtility(tmpPath=self.__sessionPath, siteId=self.__siteId, verbose=self.__verbose)
            #
            dp.setDebugMode(flag=self.__debug)
            dp.imp(inpPath)
            dp.op("chem-comp-dict-serialize")
            dp.expLog(logPath)
            dp.exp(outPathTmp)
            fSize = dp.expSize()
            if fSize > minSize:
                os.chmod(outPathTmp, 0o664)
                shutil.move(outPathTmp, ref[1])
                os.chmod(ref[1], 0o664)
                ok = True
            else:
                ok = False
            if not self.__debug:
                dp.cleanup()
            endTime0 = time.time()
            if self.__verbose:
                logger.info("Serialized dictionary file size %d  in %.2f seconds", fSize, endTime0 - startTime)
            return ok
        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("In __serializePrdDictOp")
        return False

    def __generatePrdSummaryOp(self, minSize=10):
        """Generate summary cif file and serialize it"""
        logger.info("Starting")
        startTime = time.time()

        ccSDBin = self.__pathCCDictSerial
        prdSDBin = self.__pathPrdDictSerial
        outPathTmp = self.__makeTempPath(self.__pathPrdSummary)
        outPathSerialTmp = self.__makeTempPath(self.__pathPrdSummarySerial)
        fSize = 0

        ok1 = ok2 = False
        try:
            logPath = os.path.join(self.__pathPrdDictRef, "prd-summary.log")
            dp = RcsbDpUtility(tmpPath=self.__sessionPath, siteId=self.__siteId, verbose=self.__verbose)
            dp.setDebugMode(flag=self.__debug)
            dp.imp(prdSDBin)
            dp.addInput(name="ccsdb_path", value=ccSDBin)
            dp.op("prd-summary-serialize")
            dp.expLog(logPath)
            dp.expList([outPathTmp, outPathSerialTmp])
            # If changed....
            updated = False
            f1 = DataFile(outPathTmp)
            if f1.srcFileExists():
                ok1 = True
                fSize = f1.srcFileSize()
                if fSize > minSize:
                    outPath = self.__pathPrdSummary
                    if os.path.exists(outPath):
                        if not filecmp.cmp(outPathTmp, outPath, shallow=False):
                            updated = True
                            os.unlink(outPath)
                    else:
                        updated = True

                    if updated:
                        os.chmod(outPathTmp, 0o664)
                        shutil.move(outPathTmp, outPath)
                        os.chmod(outPath, 0o664)
                    else:
                        f1.remove()
            else:
                ok1 = False

            f1 = DataFile(outPathSerialTmp)
            if updated:
                if f1.srcFileExists():
                    ok2 = True

                    updated = False
                    fSize = f1.srcFileSize()
                    if fSize > minSize:
                        outPath = self.__pathPrdSummarySerial
                        updated = False
                        if os.path.exists(outPath):
                            if not filecmp.cmp(outPathSerialTmp, outPath, shallow=False):
                                os.unlink(outPath)
                                updated = True
                        else:
                            updated = True

                    if updated:
                        os.chmod(outPathSerialTmp, 0o664)
                        shutil.move(outPathSerialTmp, outPath)
                        os.chmod(outPath, 0o664)
                    else:
                        f1.remove()
            else:
                f1.remove()
                ok2 = False

            if not self.__debug:
                dp.cleanup()
            endTime0 = time.time()

            # logger.info("Dictionary summary update ok1 %s ok2 %s", ok1, ok2)
            if self.__verbose:
                logger.info("Dictionary summary file size %d  in %.2f seconds", fSize, endTime0 - startTime)
            return ok1 and ok2

        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("In generatePrdSummaryOp")

        return False

    def __generatePrdFamilyMapOp(self, familyPathList, minSize=10):
        """Generate family mapping file"""
        logger.info("Starting")

        # We do not retain concatenation when done...
        tempFamilyFile = os.path.join(self.__pathPrdDictRef, "prd-family-temp.cif")
        outPathTmp = self.__makeTempPath(self.__pathPrdFamilyMapping)

        ok = self.concatPathList(familyPathList, tempFamilyFile)

        if ok:
            try:
                logPath = os.path.join(self.__pathPrdDictRef, "prd-family.log")
                dp = RcsbDpUtility(tmpPath=self.__sessionPath, siteId=self.__siteId, verbose=self.__verbose)
                dp.setDebugMode(flag=self.__debug)
                dp.imp(tempFamilyFile)
                dp.op("prd-family-mapping")
                dp.expLog(logPath)
                dp.exp(outPathTmp)
                fSize = dp.expSize()
                if fSize > minSize:
                    outPath = self.__pathPrdFamilyMapping
                    updated = False
                    if os.path.exists(outPath):
                        if not filecmp.cmp(outPathTmp, outPath, shallow=False):
                            os.unlink(outPath)
                            updated = True
                    else:
                        updated = True

                    if updated:
                        os.chmod(outPathTmp, 0o664)
                        shutil.move(outPathTmp, outPath)
                        os.chmod(outPath, 0o664)
                    else:
                        try:
                            os.unlink(outPathTmp)
                        except:  # noqa: E722 pylint: disable=bare-except
                            pass
                    # Remove tempFamilyFile
                    try:
                        os.unlink(tempFamilyFile)
                    except:  # noqa: E722 pylint: disable=bare-except
                        pass

                if not self.__debug:
                    dp.cleanup()
                return True

            except:  # noqa: E722 pylint: disable=bare-except
                logger.exception("Failure in generatePrdMapOp")

        return False



#
# File:  ChemRefSearchDuplicates.py
# Date:  19-Jan-2019 E. Peisach
#
# Update:
#
##
"""
Search CCD database for duplicate ligangs

"""

import sys
import datetime
import pickle
from wwpdb.utils.db.MyConnectionBase import MyConnectionBase
from wwpdb.utils.db.MyDbUtil import MyDbQuery

import logging

logger = logging.getLogger(__name__)


class ChemRefSearchDuplicates(MyConnectionBase):
    def __init__(self, verbose=True, log=sys.stderr):
        super(ChemRefSearchDuplicates, self).__init__(verbose=verbose, log=log)
        self.__verbose = verbose
        self.__lfh = log
        self.__dups = []

    def find_duplicates(self):
        """Returns a dictionary of duplcate items"""

        logger.debug("Start locating duplicate CCDs")

        self.setResource(resourceName="CC")
        ok = self.openConnection()
        if not ok:
            logger.error("Could not open connection to server")
            return None
        myq = MyDbQuery(dbcon=self._dbCon, verbose=self.__verbose, log=self.__lfh)

        sqlq = """select e.num, e.c1, c1.pdbx_release_status as status, c1.pdbx_processing_site as site,
                    c1.pdbx_modified_date as moddate,
                    e.c2, c2.pdbx_release_status as status, c2.pdbx_processing_site as site,
                    c2.pdbx_modified_date as moddate
                    from  chem_comp c1, chem_comp c2,(select count(1) as num,max(c.id) as c1,min(c.id) as c2
                    from chem_comp c, pdbx_chem_comp_descriptor  d, pdbx_chem_comp_descriptor d1,
                        pdbx_chem_comp_descriptor d2
                    where d.PROGRAM='CACTVS' and d.TYPE='SMILES_CANONICAL'
                    and d1.PROGRAM='OpenEye OEToolkits' and d1.TYPE='SMILES_CANONICAL'
                    and d2.PROGRAM='InChI' and d2.TYPE='InChI'
                    and c.pdbx_release_status != 'OBS' and c.pdbx_replaced_by = ''
                    and d.component_id=c.component_id and d1.component_id=c.component_id  and
                    d2.component_id = d.component_id
                    group by d.descriptor, d1.descriptor, d2.descriptor having count(1)>1
                        and count(distinct(c.type))<=1) e
                    where e.c1 = c1.id and e.c2 = c2.id order by e.num,e.c1,e.c2
                """

        rows = myq.selectRows(queryString=sqlq)

        self.__dups = []
        for row in rows:
            d = {
                "num": row[0],
                "cid1": row[1],
                "cid1status": row[2],
                "cid1site": row[3],
                "cid1mod": row[4],
                "cid2": row[5],
                "cid2status": row[6],
                "cid2site": row[7],
                "cid2mod": row[8],
            }
            self.__dups.append(d)

        return self.__dups

    @staticmethod
    def _dformat(dtime):
        """Formats date time for output"""
        return dtime.strftime("%d-%b-%y")

    def _rformat(self, row):
        """Formats a row for the report"""
        return "{} {:<3} {:<4} {:<4} {}   {:<3} {:<4} {:<4} {}".format(
            row["num"],
            row["cid1"],
            row["cid1site"],
            row["cid1status"],
            self._dformat(row["cid1mod"]),
            row["cid2"],
            row["cid2site"],
            row["cid2status"],
            self._dformat(row["cid2mod"]),
        )

    @staticmethod
    def _isblacklist(row):
        """If a combination is blacklisted, return explanation else Fasle"""
        blacklist = {
            "UNK,ABA": "Unknown residue",
            "DN,3DR": "Unknown residue",
            "1CR,0CR": "Enantiomers",
            "C7S,C7R": "thiophospho connection isoforms",
            "RKP,0TN": "Metallo compound different arrangements",
            "OY8,OY5": "Diastereomer carboranes",
        }

        key = "%s,%s" % (row["cid1"], row["cid2"])
        if key in blacklist:
            return blacklist[key]

        key = "%s,%s" % (row["cid2"], row["cid1"])
        if key in blacklist:
            return blacklist[key]

        return False

    @staticmethod
    def _deltadays(dtime, recent):
        """Returns True is datetime is within recent days of today"""

        if abs((dtime - datetime.datetime.today()).days) <= recent:
            return True
        return False

    def makereport(self, recentdays=0):
        """Generates a report as a list of strings"""

        rep = []
        blacklist = []
        recent = []

        rep.append("Duplicate ligands (non-obsolete, non-superceded, and non-abmiguous with identical SMILES")
        rep.append("         (both CACTVS and OpenEye) as well as InChI")
        # Error details: 2 1CR RCSB REL 04-JUN-11 0CR RCSB REL 04-JUN-11

        for row in self.__dups:
            blacklisted = self._isblacklist(row)
            if blacklisted:
                blacklist.append("{}   -- {}".format(self._rformat(row), blacklisted))
            else:
                rep.append(self._rformat(row))

            if self._deltadays(row["cid1mod"], recentdays) or self._deltadays(row["cid2mod"], recentdays):
                recent.append(self._rformat(row))

        if recent and recentdays:
            recent = ["Duplicate Ligands created in the last {} days".format(recentdays)] + recent
            rep = recent + ["", ""] + rep

        if len(blacklist) > 0:
            rep.extend(["", "", "Blacklisted known duplicates"])
            rep.extend(blacklist)

        return rep

    def savestate(self, fname):
        """Stores the current state in a pickle file so can see if any changes made"""

        with open(fname, "wb") as fout:
            # Version 1
            pickle.dump([1, self.__dups], fout)

    def restorestate(self, fname):
        """Restores state from pickle file to allow comparisons"""
        try:
            with open(fname, "rb") as fin:
                din = pickle.load(fin)
                if din[0] != 1:
                    raise ImportError("File version number wrong")
                else:
                    self.__dups = din[1]
        except IOError as _e:  # noqa: F841
            # File missing or bad format
            return False

        return True


def main():
    import argparse
    import difflib

    ch = logging.StreamHandler()
    formatter = logging.Formatter("[%(levelname)s] [%(module)s.%(funcName)s] %(message)s")
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    logger.setLevel(logging.INFO)

    parser = argparse.ArgumentParser(description="Detect duplicate ligands based on compv4 database")
    parser.add_argument("--recent", type=int, default=7, help="Highlight duplicates within n days")
    parser.add_argument("--savestate", default=None, help="Filename to load and store state")
    parser.add_argument("--skipsame", default=False, action="store_true", help="Report nothing if same")

    parsed = parser.parse_args()
    # print(parsed)

    prevdatavalid = False
    if parsed.savestate:
        refcrsd = ChemRefSearchDuplicates()
        prevdatavalid = refcrsd.restorestate(parsed.savestate)

    crsd = ChemRefSearchDuplicates()
    crsd.find_duplicates()

    # Calculate reports

    rep = crsd.makereport(recentdays=parsed.recent)

    diffs = []
    if prevdatavalid:
        prevrep = refcrsd.makereport(recentdays=parsed.recent)
        # prevrep.append("High")
        for line in difflib.ndiff(prevrep, rep):
            if line[0] in ["+", "-"]:
                diffs.append(line)
        if len(diffs):
            diffs = ["Differences since last report"] + diffs + ["", ""]
        elif parsed.skipsame:
            sys.exit(0)

    # Create final report....
    finalrep = diffs + rep

    print("\n".join(finalrep))

    if parsed.savestate:
        crsd.savestate(parsed.savestate)


if __name__ == "__main__":
    main()



##
# File:  ChemCompSearchIndexUtils.py
# Date:  1-Feb-2017 J. Westbrook
#
# Update:
#   20-Apr-2017  overhaul
#    1-May-2017  add search over lists and add range search method
#   27-May-2017  add more performant approximate string comparison
##
"""
Search index of chemical components definitions by component features.

"""

import sys
import time

import jellyfish
from operator import itemgetter

from wwpdb.utils.cc_dict_util.persist.PdbxChemCompDictIndex import PdbxChemCompDictIndex
from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppCc

import logging

logger = logging.getLogger(__name__)


class ChemCompSearchIndexUtils(object):
    def __init__(self, siteId, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        self.__siteId = siteId
        self.__cIAppCc = ConfigInfoAppCc(self.__siteId)
        self.__pathCCIndex = self.__cIAppCc.get_cc_index()
        logger.debug("ChemCompSearchIndexUtils index path %r", self.__pathCCIndex)
        #
        dIndx = PdbxChemCompDictIndex(verbose=self.__verbose, log=self.__lfh)
        self.__ccIndx = dIndx.readIndex(indexPath=self.__pathCCIndex)

    def getValue(self, ccId, key):
        try:
            return self.__ccIndx[str(ccId).upper()][key]
        except:  # noqa: E722 pylint: disable=bare-except
            pass
        return None

    def getAttributeValueList(self, myIdList, colList, dspList):
        """For the input id list return a list of dictionaries populated with
        the key values in colList.  Return dictionary keys are taken from
        the corresponding elements of dspList.
        """
        logger.debug("getAttributeValueList myIdList %r", myIdList)
        logger.debug("getAttributeValueList colList %r", colList)
        logger.debug("getAttributeValueList dspList %r", dspList)
        logger.debug("getAttributeValueList length index %r", len(self.__ccIndx))
        rowList = []
        for myId in myIdList:
            d = self.__ccIndx[str(myId).upper()]
            dd = {}
            if self.__debug:
                logger.debug("getAttributeValueList myId %r d %r", myId, d)
            for ii, col in enumerate(colList):
                dd[dspList[ii]] = d[col]
            rowList.append(dd)
        return rowList

    def searchIndexAll(self, key):
        rD = {}
        for ccId, d in self.__ccIndx.items():
            try:
                refV = d[key]
                mL = self.searchIndex(refV, key)
                if len(mL) > 0:  # THIS WAS > 1 - so seach for single row failed
                    rD[ccId] = mL
            except Exception as e:
                logger.exception("Index search failing for key %r  %r", key, str(e))
        return rD

    def searchIndex(self, target, key):
        """ """
        idList = []
        try:
            logger.debug("Search index key %r target %r", key, target)
            startTime = time.time()
            #
            for ccId, d in self.__ccIndx.items():
                # refV = d.get(key, '')
                refV = d[key]
                if isinstance(refV, (list, tuple, set)):
                    for tt in refV:
                        if target == tt:
                            idList.append(ccId)
                            continue
                else:
                    if target == refV:
                        idList.append(ccId)

            endTime = time.time()
            logger.debug("SearchIndex %r %r  match list for length %d in (%.4f seconds)", target, key, len(idList), endTime - startTime)
        except Exception as e:
            logger.exception("Index search failing for key %r target %r %r", key, target, str(e))
        #
        return idList

    def searchIndexSubstring(self, target, key):
        """ """
        idList = []
        try:
            startTime = time.time()
            # Upper and lower bounds for element counts in the input formula.
            #
            for ccId, d in self.__ccIndx.items():
                # refV = d.get(key, '')
                refV = d[key]
                refV = refV if refV is not None else ""
                if isinstance(refV, (list, tuple, set)):
                    for tt in refV:
                        if target in tt:
                            idList.append(ccId)
                            continue
                else:
                    if target in refV:
                        idList.append(ccId)

            endTime = time.time()
            logger.debug("SearchIndexSubstring %r %r  match list for length %d in (%.4f seconds)", target, key, len(idList), endTime - startTime)
        except Exception as e:
            logger.exception("Index substring search failing for key %r target %r %r", key, target, str(e))
        #
        return idList

    def searchIndexRange(self, target, key):
        """ """
        idList = []
        try:
            logger.debug("Search index range  key %r target range %r", key, target)
            startTime = time.time()
            #
            bL = [t.strip() for t in target.split()]
            lowB = float(bL[0])
            upperB = float(bL[1])
            for ccId, d in self.__ccIndx.items():
                # refV = d.get(key, '')
                try:
                    refV = float(str(d[key]))
                except:  # noqa: E722 pylint: disable=bare-except
                    logger.debug("Conversion failure for %r %r %r", key, ccId, d[key])
                    continue
                if lowB <= refV and refV < upperB:
                    idList.append(ccId)

            endTime = time.time()
            logger.debug("SearchIndex %r %r  match list for length %d in (%.4f seconds)", target, key, len(idList), endTime - startTime)
        except Exception as e:
            logger.exception("Index search failing for key %r target %r %r", key, target, str(e))
        #
        return idList

    def parseFormulaInput(self, inpTarget):
        """Standardize the input formula target and return a
        formula string - El## El##  and element count dictionary -
        """
        #
        ed = {}
        target = inpTarget
        try:
            #
            lt = inpTarget.upper().split(" ")
            # standardize counts - add missing unit count
            ll = []
            for el in lt:
                if not el[-1].isdigit():
                    el += "1"
                ll.append(el)

            tt = "".join(ll)
            if len(tt) > 0:
                aa = tt[0]
                for i in range(1, len(tt)):
                    if tt[i - 1].isdigit() and tt[i].isalpha():
                        aa += " "
                    aa += tt[i]
                target = aa
            #
            ed = {}
            ecL = target.upper().split(" ")
            for ec in ecL:
                if len(ec) < 2:
                    logger.info("parseFormulaInput - %r %r format error ", target, ecL)
                    continue
                if ec[1].isalpha():
                    ed[ec[0:2]] = int(ec[2:])
                else:
                    ed[ec[0:1]] = int(ec[1:])
            #
        except:  # noqa: E722 pylint: disable=bare-except
            ed = {}
            logger.exception("parseFormulaInput failed")

        return target, ed

    def searchFormulaBounded(self, elementCounts=None, upperOffset=2, lowerOffset=2, excludeH=False):
        """Bounded formula search - return a list of component identifiers satisfying
        the bounded element count criteria on the input formula.  The input formula
        is represented as a dictionary of element type and count. Upper and lower offsets are
        provided to specify upper and lower bounds for the element count comparison.

        """
        if elementCounts is None:
            elementCounts = {"C": 20, "Cl": 4}
        idList = []
        if len(elementCounts) < 1:
            return idList
        try:
            startTime = time.time()
            # Upper and lower bounds for element counts in the input formula.
            #
            offsetU = upperOffset
            offsetL = lowerOffset
            #
            for ccId, d in self.__ccIndx.items():
                refC = d["typeCounts"]
                if len(refC) < 1:
                    continue
                #
                mOk = True
                for etype, cnt in elementCounts.items():
                    typeU = etype.upper()
                    if excludeH and typeU in ["H", "D", "T"]:
                        continue
                    if typeU in refC:
                        # test out of bounds -
                        if (cnt < refC[typeU] - offsetL) or (cnt > refC[typeU] + offsetU):
                            mOk = False
                            break
                    else:
                        mOk = False
                        break
                if mOk:
                    idList.append(ccId)

            endTime = time.time()
            logger.info("FormulaBounded match list for length %d (%.3f seconds)", len(idList), endTime - startTime)

        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("Formula search failing")

        return idList

    def searchFormulaSubset(self, elementCounts=None, excludeH=False):
        """The input formula is represented as a dictionary of element type and count.
        Search for an reference formula that are an exact subset of the target.

        """
        if elementCounts is None:
            elementCounts = {"C": 20, "Cl": 4}
        idList = []
        if len(elementCounts) < 1:
            return idList
        try:
            startTime = time.time()
            #
            for ccId, d in self.__ccIndx.items():
                refC = d["typeCounts"]
                if len(refC) < 1:
                    continue
                #
                mOk = True
                for etype, cnt in elementCounts.items():
                    typeU = etype.upper()
                    if excludeH and typeU in ["H", "D", "T"]:
                        continue
                    if typeU in refC:
                        if cnt != refC[typeU]:
                            mOk = False
                            break
                    else:
                        mOk = False
                        break
                if mOk:
                    idList.append(ccId)

            endTime = time.time()
            logger.info("Formula match list for length %d (%.3f seconds)", len(idList), endTime - startTime)

        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("Formula search failing")

        #
        return idList

    def searchFormulaExact(self, elementCounts=None, excludeH=False):
        """The input formula is represented as a dictionary of element type and count.
        Search for an reference formula that is an exact match of the target.

        """
        if elementCounts is None:
            elementCounts = {"C": 20, "Cl": 4}

        logger.info("ElementCounts %r excludeH %r", elementCounts, excludeH)
        idList = []
        if len(elementCounts) < 1:
            return idList
        try:
            startTime = time.time()
            #
            for ccId, d in self.__ccIndx.items():
                refC = d["typeCounts"]
                if len(refC) < 1:
                    continue
                # Exact match case -
                if not excludeH and len(refC) != len(elementCounts):
                    continue
                elif excludeH:
                    rs = set(refC.keys()) - set(["H", "T", "D"])
                    ts = set(elementCounts.keys()) - set(["H", "T", "D"])
                    if rs != ts:
                        continue
                    # if len(rs) != len(ts):
                    #    continue
                #

                mOk = True
                for etype, cnt in refC.items():
                    #
                    typeU = etype.upper()
                    if excludeH and typeU in ["H", "D", "T"] and len(refC) > 1:
                        continue
                    if typeU in elementCounts:
                        if cnt != elementCounts[typeU]:
                            mOk = False
                            break
                    else:
                        mOk = False
                        break
                if mOk:
                    idList.append(ccId)

            endTime = time.time()
            logger.info("Formula match list for length %d (%.3f seconds)", len(idList), endTime - startTime)

        except:  # noqa: E722 pylint: disable=bare-except
            logger.exception("Formula search failing")
        #
        return idList

    ##
    ##

    def searchEditDistance(self, target, key, DIST_TYPE="JARO_WINKLER"):
        """ """
        if DIST_TYPE == "JARO":
            return self.__searchEditDistanceJaro(target, key, cutOff=0.75)
        elif DIST_TYPE == "JARO_WINKLER":
            return self.__searchEditDistanceJaroWinkler(target, key, cutOff=0.75)
        elif DIST_TYPE == "LEV":
            return self.__searchEditDistanceDmLev(target, key, cutOff=0.5)
        else:
            return []

    def __searchEditDistanceJaro(self, target, key, cutOff=0.7):
        """ """
        cidList = []
        try:
            logger.debug("Edit distance index search key %r target %r", key, target)
            startTime = time.time()
            #
            idList = []
            for ccId, d in self.__ccIndx.items():
                refV = d[key]
                if isinstance(refV, (list, tuple, set)):
                    for tt in refV:
                        ed = jellyfish.jaro_distance(target.decode("unicode-escape"), tt.decode("unicode-escape"))
                        if ed > cutOff:
                            idList.append((ccId, ed))
                            break
                else:
                    ed = jellyfish.jaro_distance(target.decode("unicode-escape"), refV.decode("unicode-escape"))
                    if ed > cutOff:
                        idList.append((ccId, ed))

            idList = sorted(idList, key=itemgetter(1), reverse=True)
            cidList = [c for (c, r) in idList]
            endTime = time.time()
            logger.debug("Edit distance result %r %r  match list for length %d in (%.4f seconds)", target, key, len(idList), endTime - startTime)
        except Exception as e:
            logger.exception("Edit distance search failing for key %r target %r %r", key, target, str(e))
        #
        return cidList

    def __searchEditDistanceJaroWinkler(self, target, key, cutOff=0.7):
        """ """
        cidList = []
        try:
            logger.debug("Edit distance index search key %r target %r", key, target)
            startTime = time.time()
            #
            idList = []
            for ccId, d in self.__ccIndx.items():
                refV = d[key]
                if isinstance(refV, (list, tuple, set)):
                    for tt in refV:
                        ed = jellyfish.jaro_winkler(target.decode("unicode-escape"), tt.decode("unicode-escape"))
                        if ed > cutOff:
                            idList.append((ccId, ed))
                            break
                else:
                    ed = jellyfish.jaro_winkler(target.decode("unicode-escape"), refV.decode("unicode-escape"))
                    if ed > cutOff:
                        idList.append((ccId, ed))

            idList = sorted(idList, key=itemgetter(1), reverse=True)
            cidList = [c for (c, r) in idList]
            endTime = time.time()
            logger.debug("Edit distance result %r %r  match list for length %d in (%.4f seconds)", target, key, len(idList), endTime - startTime)
        except Exception as e:
            logger.exception("Edit distance search failing for key %r target %r %r", key, target, str(e))
        #
        return cidList

    def __searchEditDistanceDmLev(self, target, key, cutOff=0.7):
        """ """
        cidList = []
        try:
            logger.debug("Edit distance index search key %r target %r", key, target)
            startTime = time.time()
            #
            idList = []
            for ccId, d in self.__ccIndx.items():
                refV = d[key]
                if isinstance(refV, (list, tuple, set)):
                    for tt in refV:
                        ed = self.__editDistanceNormJF(target.decode("unicode-escape"), tt.decode("unicode-escape"))
                        if ed > cutOff:
                            idList.append((ccId, ed))
                            break
                else:
                    if self.__editDistanceNormJF(target.decode("unicode-escape"), refV.decode("unicode-escape")) > cutOff:
                        idList.append((ccId, ed))
            idList = sorted(idList, key=itemgetter(1), reverse=True)
            cidList = [c for (c, r) in idList]
            endTime = time.time()
            logger.debug("Edit distance result %r %r  match list for length %d in (%.4f seconds)", target, key, len(idList), endTime - startTime)
        except Exception as e:
            logger.exception("Edit distance search failing for key %r target %r %r", key, target, str(e))
        #
        return cidList

    def __editDistanceNormJF(self, s1, s2):
        """
        Compute the Normalized Damerau-Levenshtein distance between two given
        strings (s1 and s2)
        """
        try:
            maxLen = max(len(s1), len(s2))
            ed = jellyfish.damerau_levenshtein_distance(s1, s2)  # pylint: disable=no-member
            edN = float(maxLen - ed) / float(maxLen)
            # logging.debug("s1 %r s2 %r maxLen %r ed %f edN %f " % (s1, s2, maxLen, ed, edN))
            return edN
        except Exception as e:
            logger.exception("Edit distance norm s1 %r s2 %r %r", s1, s2, str(e))
        return 0

    # def __editDistanceNorm(self, s1, s2):
    #     """
    #     Compute the Normalized Damerau-Levenshtein distance between two given
    #     strings (s1 and s2)
    #     """
    #     try:
    #         maxLen = max(len(s1), len(s2))
    #         ed = self.__editDistance(s1, s2)
    #         edN = float(maxLen - ed) / float(maxLen)
    #         # logging.debug("s1 %r s2 %r maxLen %r ed %f edN %f " % (s1, s2, maxLen, ed, edN))
    #         return edN
    #     except Exception as e:
    #         logger.exception("Edit distance norm s1 %r s2 %r %r" % (s1, s2, str(e)))
    #     return 0

    # def __editDistance(self, s1, s2):
    #     """
    #     Compute the Damerau-Levenshtein distance between two given
    #     strings (s1 and s2)
    #     """
    #     d = {}
    #     lenstr1 = len(s1)
    #     lenstr2 = len(s2)
    #     for i in range(-1, lenstr1 + 1):
    #         d[(i, -1)] = i + 1
    #     for j in range(-1, lenstr2 + 1):
    #         d[(-1, j)] = j + 1

    #     for i in range(0, lenstr1):
    #         for j in range(0, lenstr2):
    #             if s1[i] == s2[j]:
    #                 cost = 0
    #             else:
    #                 cost = 1
    #             d[(i, j)] = min(
    #                 d[(i - 1, j)] + 1,  # deletion.
    #                 d[(i, j - 1)] + 1,  # insertion
    #                 d[(i - 1, j - 1)] + cost,  # substitution
    #             )
    #             if i > 1 and j > 1 and s1[i] == s2[j - 1] and s1[i - 1] == s2[j]:
    #                 d[(i, j)] = min(d[(i, j)], d[i - 2, j - 2] + cost)  # transposition

    #     return d[lenstr1 - 1, lenstr2 - 1]



##
# File:  ChemRefDataDepict.py
# Date:  29-Jan-2013
#
# Updates:
#
# 10-Feb-2013 jdw provide encapsulating methods for reusable page sections.
# 12-Fev-2013 jdw share includes with static application pages.
#
##
"""
Base class for HTML depictions containing common HTML constructs using
JQuery UI and SuperFish menus.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.02"

import os
import sys


class ChemRefDataDepict(object):
    """Base class for HTML depictions contain definitions of common constructs."""

    def __init__(self, includePath=None, verbose=False, log=sys.stderr):  # pylint: disable=unused-argument
        """

        :param `verbose`:  boolean flag to activate verbose logging.
        :param `log`:      stream for logging.

        """
        self.__includePath = includePath
        #
        self.__includeHeadList = ["head_common.html"]
        self.__includeTopList = ["page_header.html", "topmenu.html"]
        self.__includeBottomList = ["page_footer.html"]

        #
        #
        self._pragma = r"""<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">"""
        self._pragmaXhtml = r"""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">"""
        #
        self.__meta = r"""<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />"""

        self._jQueryGenericInclude = r"""

<title>Peptide Reference Dictionary Tools</title>
<link rel="stylesheet" type="text/css"  media="all" href="/birdmodule/styles/general.css" />
<link rel="stylesheet" type="text/css"  media="all" href="/birdmodule/styles/thrColLiqHdr.css" />
<link rel="stylesheet" type="text/css"  media="all" href="/birdmodule/styles/superfish.css"  />
<link rel="stylesheet" type="text/css"  media="all" href="/birdmodule/styles/ui-lightness/jquery-ui-1.8.6.custom.css" />
<script type="text/javascript" src="/birdmodule/js/local/my-utils-1.0.js"></script>
<script type="text/javascript" src="/birdmodule/js/jquery-1.4.4.min.js"></script>
<!--[if IE]><script type="text/javascript" src="/birdmodule/js/excanvas.compiled.js" charset="utf-8"></script><![endif]-->
<script type="text/javascript" src="/birdmodule/js/jquery.hoverIntent.minified.js"></script>
<script type="text/javascript" src="/birdmodule/js/jquery.bgiframe.min.js"></script>
<script type="text/javascript" src="/birdmodule/js/superfish.js"></script>
<script type="text/javascript" src="/birdmodule/js/jquery-ui-1.8.6.custom.min.js"></script>
<script type="text/javascript" src="/birdmodule/js/jquery.corner.js"></script>
"""

        #
        self._jQueryGenericIncludeN = r"""

        <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
        <title>Chemical Reference Data Management Module</title>
        <link rel="stylesheet" type="text/css"  media="all" href="/chem_ref_data_ui/styles/general.css" />
        <link rel="stylesheet" type="text/css"  media="all" href="/chem_ref_data_ui/styles/thrColLiqHdr.css" />
        <link rel="stylesheet" type="text/css"  media="all" href="/chem_ref_data_ui/styles/superfish.css"  />

        <link rel="stylesheet" type="text/css"  media="all" href="/styles/themes/south-street/jquery-ui-1.8.2.custom.min.css" />

        <script type="text/javascript" src="/chem_ref_data_ui/js/my-utils-1.0.js"></script>
        <script type="text/javascript" src="/js/jquery/core/jquery.min.js"></script>
        <script type="text/javascript" src="/js/jquery/ui/jquery-ui.custom.min.js"></script>

        <script type="text/javascript" src="/chem_ref_data_ui/js/jquery/plugins/jquery.form.js"></script>
        <script type="text/javascript" src="/chem_ref_data_ui/js/jquery/plugins/jquery.url.js"></script>


        <link rel="stylesheet" type="text/css" media="all" href="/chem_ref_data_ui/styles/chemrefCommon.css" />



        <script type="text/javascript" src="/chem_ref_data_ui/js/jquery/plugins/jquery.hoverIntent.minified.js"></script>
        <script type="text/javascript" src="/chem_ref_data_ui/js/jquery/plugins/jquery.bgiframe.min.js"></script>
        <script type="text/javascript" src="/chem_ref_data_ui/js/jquery/plugins/superfish.js"></script>

        <script type="text/javascript" src="/chem_ref_data_ui/js/jquery/ui/js/jquery-ui-1.8.20.custom.min.js"></script>
        <script type="text/javascript" src="/chem_ref_data_ui/js/jquery/plugins/jquery.corner.js"></script>

        """

        # <script type="text/javascript" src="/chem_ref_data_ui/js/chemrefCommon.js"></script>

        # ---------------
        self._menuCommonIncludeInline = r"""
        <!-- begin #header-->
        <div id="header">
        <div id="logo"><img src="/images/wwpdb_logo.gif" width="187" height="58" alt="logo" /> </div>
        <div id="headerCont">
                <h1>Chemical Reference Data Management Module</h1>
                <!-- <span id="help_standalone_module" class="ui-icon ui-icon-info fltrgt"></span> -->
                </div>
        <br class="clearfloat" />
        </div>
        <!-- #header ends above-->
        <!-- begin #menu-->
       <div id="menu">
        <ul id="top-menu-options" class="sf-menu">
                <li class="current"> <a href="chemref_index.html" class="first">Home</a> </li>
                <li> <a  href="chemref_search.html">Search</a> </li>
                <li> <a  href="chemref_admin.html">Admin</a>  </li>
                <li> <a  href="chemref_help.html">Help</a> </li>
                <li> <a  href="mailto:jwest@rcsb.rutgers.edu">Feedback</a>  </li>
       </ul>
       </div>
       <!-- #menu ends above-->
       <br class="clearfloat" />
       <br/>
       <!-- END HEADER INCLUDE FILE -->
       """

        #
        self._trailer = r"""
        """
        #
        self._footer_test = r"""
        <!-- BEGIN FOOTER -->
        <br class="clearfloat" />
        <div id="footer">
        <p>&copy; 2013 wwPDB Chemical Reference Data Management Module V0.01</p>
        </div>
        <!-- END   FOOTER -->

        """

    #
    def appDocType(self):
        return self._pragmaXhtml

    def appMetaTags(self):
        return self.__meta

    def appPageTop(self, title=None, scriptPathList=None, cssPathList=None):
        """Return the application specific top of page boiler plate -"""
        oL = []
        oL.append(self.appDocType())
        oL.append('<html xmlns="http://www.w3.org/1999/xhtml">')
        oL.append("<head>")
        if title is not None:
            oL.append("<title>%s</title>" % title)
        try:
            for fn in self.__includeHeadList:
                pth = os.path.join(self.__includePath, fn)
                ifh = open(pth, "r")
                oL.append(ifh.read())
                ifh.close()
        except:  # noqa: E722 pylint: disable=bare-except
            pass

        try:
            for pth in cssPathList:
                oL.append('<link rel="stylesheet" type="text/css" media="all" href="%s" />' % pth)
        except:  # noqa: E722 pylint: disable=bare-except
            pass

        try:
            for pth in scriptPathList:
                oL.append('<script type="text/javascript" src="%s"></script>' % pth)
        except:  # noqa: E722 pylint: disable=bare-except
            pass
        oL.append("</head>")

        oL.append('<body class="oneColLiqHdr">')
        try:
            for fn in self.__includeTopList:
                pth = os.path.join(self.__includePath, fn)
                ifh = open(pth, "r")
                oL.append(ifh.read())
                ifh.close()
        except:  # noqa: E722 pylint: disable=bare-except
            pass

        oL.append('<br class="clearfloat" />')
        oL.append("<br />")
        oL.append('<div id="mainContent">')
        return oL

    def appPageBottom(self):
        """Return the application specific bottom of page boiler plate -

        </div> <!-- END of mainContent  -->
        <!--#include virtual="includes/page_footer.html"-->
        </body>
        </html>

        """
        oL = []
        oL.append("</div> <!-- END of mainContent  -->")
        try:
            for fn in self.__includeBottomList:
                pth = os.path.join(self.__includePath, fn)
                ifh = open(pth, "r")
                oL.append(ifh.read())
                ifh.close()

        except:  # noqa: E722 pylint: disable=bare-except
            pass
        oL.append("</body>")
        oL.append("</html>")
        return oL



##
# File:  ChemRefDataDepictBootstrap.py
# Date:  18-Feb-2013
#
# Updates:
#
#
##
"""
Base class for HTML depictions containing common HTML constructs using
BootStrap and

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.02"

import os
import sys


class ChemRefDataDepictBootstrap(object):
    """Base class for HTML depictions contain definitions of common constructs
    using HTML5 and Bootstrap CSS -

    """

    def __init__(self, includePath=None, verbose=False, log=sys.stderr):  # pylint: disable=unused-argument
        """

        :param `verbose`:  boolean flag to activate verbose logging.
        :param `log`:      stream for logging.

        """
        self.__includePath = includePath
        #
        # Within the <head></head> section
        self.__includeHeadList = ["head_common_bs.html"]
        # At the beginning of the <body>
        self.__includeTopList = ["page_header_bs.html"]
        #
        self.__includeBottomList = ["page_footer_bs.html"]
        #
        self.__includeJavaScriptList = ["page_javascript_bs.html"]
        #
        self.__meta = r"""<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />"""

    #
    def appDocType(self):
        return "<!DOCTYPE html>"

    def appMetaTags(self):
        return self.__meta

    def appPageTop(self, title=None, cssPathList=None):
        """Return the application specific top of page boiler plate -"""
        oL = []
        oL.append("<!DOCTYPE html>")
        oL.append('<html lang="en"')
        oL.append("<head>")
        try:
            for fn in self.__includeHeadList:
                pth = os.path.join(self.__includePath, fn)
                ifh = open(pth, "r")
                oL.append(ifh.read())
                ifh.close()
        except:  # noqa: E722 pylint: disable=bare-except
            pass

        #
        # include application specific CSS files --
        try:
            for pth in cssPathList:
                oL.append('<link rel="stylesheet" type="text/css" media="all" href="%s" />' % pth)
        except:  # noqa: E722 pylint: disable=bare-except
            pass

        if title is not None:
            oL.append("<title>%s</title>" % title)

        oL.append("</head>")
        #
        oL.append("<body>")

        oL.append('<div id="wrap">')
        try:
            for fn in self.__includeTopList:
                pth = os.path.join(self.__includePath, fn)
                ifh = open(pth, "r")
                oL.append(ifh.read())
                ifh.close()
        except:  # noqa: E722 pylint: disable=bare-except
            pass

        oL.append('<div class="container">')

        oL.append("<!-- # Application content begins here -->")

        return oL

    def appPageBottom(self, scriptPathList=None):
        """Return the application specific bottom of page boiler plate -

        <!--#include virtual="includes/page_footer.html"-->
        </div> <!-- end container -->

        <!--#include virtual="includes/page_javascript_bs.html"-->
        <script ... app specific script files > </script>

        </body>
        </html>

        """
        oL = []
        oL.append("</div> <!-- end of application content  -->")
        oL.append("</div> <!-- end of wrapper  -->")

        # oL.append('</div> <!-- end container -->')
        try:
            for fn in self.__includeJavaScriptList:
                pth = os.path.join(self.__includePath, fn)
                ifh = open(pth, "r")
                oL.append(ifh.read())
                ifh.close()
        except:  # noqa: E722 pylint: disable=bare-except
            pass

        try:
            for pth in scriptPathList:
                oL.append('<script type="text/javascript" src="%s"></script>' % pth)
        except:  # noqa: E722 pylint: disable=bare-except
            pass

        try:
            for fn in self.__includeBottomList:
                pth = os.path.join(self.__includePath, fn)
                ifh = open(pth, "r")
                oL.append(ifh.read())
                ifh.close()
        except:  # noqa: E722 pylint: disable=bare-except
            pass

        oL.append("</body>")
        oL.append("</html>")
        return oL



##
# File:  DownloadUtils.py
# Date:  12-Feb-2013
#
# Updated:
# 14-Feb-2013 jdw reorganize api  method, add fetchFile and fetchId methods.
#
"""
Common methods for managing session files and chemical reference data files for
download.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import sys
import os
import os.path
import shutil
import logging

from wwpdb.io.locator.ChemRefPathInfo import ChemRefPathInfo

logger = logging.getLogger(__name__)


class DownloadUtils(object):
    """Common methods for managing session files and chemical reference data files for
    download.
    """

    #

    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """Input request object is used to determine session context."""
        self.__verbose = verbose
        self.__lfh = log
        self.__reqObj = reqObj
        #
        self.__siteId = self.__reqObj.getValue("WWPDB_SITE_ID")
        self.__crPI = ChemRefPathInfo(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        #
        self.__sessionId = self.__reqObj.getSessionId()
        self.__sessionPath = self.__reqObj.getSessionPath()
        #
        self.__sessionDir = "sessions"
        self.__downloadDir = "downloads"
        #
        self.__downloadDirPath = os.path.join(self.__sessionPath, self.__sessionId, self.__downloadDir)
        self.__webDownloadDirPath = os.path.join("/", self.__sessionDir, self.__sessionId, self.__downloadDir)
        self.__webDownloadFilePath = None
        #
        self.__targetFilePath = None
        self.__targetFileName = None
        self.__downloadFilePath = None
        self.__setup()

    def __setup(self):
        if not os.access(self.__downloadDirPath, os.W_OK):
            os.makedirs(self.__downloadDirPath, 0o755)

    def getIdType(self, idCode):
        return self.__crPI.getIdType(idCode=idCode)

    def getIdFromFileName(self, filePath):
        return self.__crPI.assignIdCodeFromFileName(filePath)

    def fetchId(self, idCode):
        filePath = self.__crPI.getFilePath(idCode=idCode)
        return self.fetchFile(filePath)

    def fetchFile(self, filePath):
        """Save input file in session download"""
        try:
            if self.__verbose:
                logger.info("+DownloadUtils.fetchFile() + target file path %s", filePath)
            self.__targetFilePath = filePath
            (_pth, self.__targetFileName) = os.path.split(self.__targetFilePath)
            self.__downloadFilePath = os.path.join(self.__downloadDirPath, self.__targetFileName)
            shutil.copyfile(self.__targetFilePath, self.__downloadFilePath)
            self.__webDownloadFilePath = os.path.join(self.__webDownloadDirPath, self.__targetFileName)
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            if self.__verbose:
                logger.info("+DownloadUtils.fetchFile() + failed for file %s", filePath)
            logger.exception("Failure in fetchFile")
        return False

    def getWebPath(self):
        """ """
        return self.__webDownloadFilePath

    def getDownloadPath(self):
        """ """
        return self.__downloadFilePath

    def getAnchorTag(self, label=None, target="_blank"):
        """Return the anchor tag corresponding the current download file selection."""
        if label is not None and len(label) > 0:
            txt = label
        else:
            txt = self.__targetFileName

        return '<a href="%s" target="%s">%s</a>' % (self.__webDownloadFilePath, target, txt)



##
# File:  ChemRefSearch.py
# Date:  22-Apr-2010 J. Westbrook
#
# Update:
#
##
"""
Chemical reference search general implementation -

"""

import sys

from wwpdb.apps.chem_ref_data.search.ChemRefSearchBase import ChemRefSearchBase
from wwpdb.apps.chem_ref_data.search.ChemRefSearchDef import ChemRefSearchDef


class ChemRefSearch(ChemRefSearchBase):
    def __init__(self, siteId=None, verbose=False, log=sys.stderr):
        super(ChemRefSearch, self).__init__(siteId=siteId, verbose=verbose, log=log)
        #
        self.set(ChemRefSearchDef._displayTypeDict, ChemRefSearchDef._keyDict, ChemRefSearchDef._searchTypeDict, ChemRefSearchDef._queryTypeDict)
        #
